<?php
/*
Plugin Name: Mimoza SEO Link Listeleme Eklentisi
Description: WordPress sitenizdeki tüm yazı ve sayfa URL'lerini listeler ve SEO analizleri yapar. Başlıklar, anahtar kelimeler, iç ve dış bağlantılar ile SEO puanı analizleri sunar. Kırık linkleri tespit eder ve tüm bu verileri CSV dosyası olarak dışa aktarır.
Version: 1.1
Author: Mimoza Bilişim

### Özellikler:
1. Tüm yazı ve sayfa URL'lerini admin panelinden listeleyin.
2. SEO puanı hesaplamaları yapın (başlık, meta açıklama, alt etiketleri vb.).
3. İç ve dış bağlantıları analiz edin, eksik veya fazla olanlar için öneriler alın.
4. Kırık linkleri tespit edin ve listeleyin.
5. Tüm bu sonuçları CSV formatında dışa aktarın.
*/

if (!defined('ABSPATH')) {
    exit; // Doğrudan erişimi engelle
}

// Admin panelinde menüye eklenti ekleme
add_action('admin_menu', 'mimoza_seo_link_menu');

function mimoza_seo_link_menu() {
    // Ana menü oluşturulması
    add_menu_page(
        'SEO Link Listeleme', // Sayfa Başlığı
        'SEO Linkler',        // Menü Başlığı
        'manage_options',     // Kullanıcı Yetkisi
        'mimoza-seo-link',    // Menü Slug
        'mimoza_seo_link_page',// Gösterilecek Fonksiyon
        'dashicons-admin-links', // Menü İkonu
        6  // Menüdeki sırası
    );

    // Kırık Linkler alt menüsünü ekle
    add_submenu_page(
        'mimoza-seo-link',    // Ana menü slug
        'Kırık Linkler',      // Sayfa başlığı
        'Kırık Linkler',      // Menüde görünecek başlık
        'manage_options',     // Kullanıcı yetkisi
        'mimoza-broken-links',// Slug
        'mimoza_broken_links_page' // Gösterilecek fonksiyon
    );
}

// Export butonu ekleyelim
function mimoza_seo_link_page() {
    ?>
    <div class="wrap">
        <h1>SEO Link Listeleme ve Meta Açıklamaları</h1>
        <p>Bu sayfa, tüm yazı ve sayfalarınızın meta açıklamalarını listelemenizi ve düzenlemenizi sağlar.</p>
        <form method="post" action="">
            <input type="submit" name="mimoza_export_csv" class="button button-primary" value="Export to CSV">
        </form>
        <?php mimoza_seo_link_list(); ?>
    </div>
    <?php

    // Export işlemi tetiklendi mi?
    if (isset($_POST['mimoza_export_csv'])) {
        mimoza_export_to_csv();
    }
}

// CSV dışa aktarma fonksiyonu
function mimoza_export_to_csv() {
    global $wpdb;

    // Tüm yazı ve sayfa linklerini sorgula
    $posts = $wpdb->get_results("
        SELECT ID, post_title, post_name, post_type 
        FROM $wpdb->posts 
        WHERE post_status = 'publish' 
        AND (post_type = 'post' OR post_type = 'page')
    ");

    if ($posts) {
        // CSV başlıkları
        $csv_data = array(
            array('ID', 'Başlık', 'URL', 'Meta Açıklaması', 'SEO Puanı', 'Durum')
        );

        // Her yazı veya sayfa için verileri al
        foreach ($posts as $post) {
            $post_url = get_permalink($post->ID);
            $meta_desc = get_post_meta($post->ID, '_yoast_wpseo_metadesc', true); // Meta açıklama
            $seo_score = mimoza_seo_score($post->ID); // SEO puanı
            $is_broken = mimoza_check_broken_link($post_url) ? 'Geçerli' : 'Kırık'; // Kırık link durumu

            // CSV'ye eklenecek veriler
            $csv_data[] = array(
                $post->ID,
                $post->post_title,
                $post_url,
                $meta_desc,
                $seo_score,
                $is_broken
            );
        }

        // CSV dosyasını başlat
        header('Content-Type: text/csv');
        header('Content-Disposition: attachment;filename="mimoza_seo_export.csv"');
        $output = fopen('php://output', 'w');

        // CSV satırlarını yazdır
        foreach ($csv_data as $row) {
            fputcsv($output, $row);
        }
        fclose($output);
        exit;
    } else {
        echo '<p>Hiçbir yazı veya sayfa bulunamadı.</p>';
    }
}

// Kırık Linkler Sayfası
function mimoza_broken_links_page() {
    ?>
    <div class="wrap">
        <h1>Kırık Linkler</h1>
        <p>Bu sayfa, sitenizdeki kırık linkleri listeler.</p>
        <?php mimoza_list_broken_links(); ?>
    </div>
    <?php
}

// Kırık linkleri listeleme fonksiyonu
function mimoza_list_broken_links() {
    global $wpdb;

    // Tüm yazı ve sayfa linklerini sorgula
    $posts = $wpdb->get_results("
        SELECT ID, post_title, post_name, post_type 
        FROM $wpdb->posts 
        WHERE post_status = 'publish' 
        AND (post_type = 'post' OR post_type = 'page')
    ");

    if ($posts) {
        echo '<table class="widefat fixed">';
        echo '<thead><tr><th>ID</th><th>Başlık</th><th>URL</th><th>Durum</th></tr></thead>';
        echo '<tbody>';
        foreach ($posts as $post) {
            $post_url = get_permalink($post->ID);

            // Kırık link kontrolü
            $is_broken = mimoza_check_broken_link($post_url) ? 'Geçerli' : 'Kırık';

            // Sadece kırık linkleri göster
            if ($is_broken === 'Kırık') {
                echo '<tr>';
                echo '<td>' . $post->ID . '</td>';
                echo '<td>' . $post->post_title . '</td>';
                echo '<td><a href="' . esc_url($post_url) . '" target="_blank">' . esc_html($post_url) . '</a></td>';
                echo '<td>' . esc_html($is_broken) . '</td>'; // Kırık link durumu
                echo '</tr>';
            }
        }
        echo '</tbody>';
        echo '</table>';
    } else {
        echo '<p>Hiçbir yazı veya sayfa bulunamadı.</p>';
    }
}

// Kırık linkleri kontrol etmek için fonksiyon
function mimoza_check_broken_link($url) {
    $response = wp_remote_get($url);

    if (is_wp_error($response)) {
        return false; // Eğer bir hata varsa link kırık
    }

    $status_code = wp_remote_retrieve_response_code($response);
    return $status_code === 200; // 200 durumu başarılı çalıştığını gösterir
}


// SEO puanlama fonksiyonu
function mimoza_seo_score($post_id) {
    $score = 100; // Başlangıç skoru

    // Başlık kontrolü (Anahtar kelime başlıkta mı?)
    $post_title = get_the_title($post_id);
    $focus_keyword = 'hedef anahtar kelime'; // Anahtar kelimeyi burada belirt

    if (strpos(strtolower($post_title), strtolower($focus_keyword)) === false) {
        $score -= 20; // Anahtar kelime yoksa puanı düşür
    }

    // Meta açıklama kontrolü (Uzunluk ve anahtar kelime)
    $meta_desc = get_post_meta($post_id, '_yoast_wpseo_metadesc', true);
    if (strlen($meta_desc) < 120 || strlen($meta_desc) > 160) {
        $score -= 20; // Meta açıklama çok kısa veya çok uzunsa puanı düşür
    }
    if (strpos(strtolower($meta_desc), strtolower($focus_keyword)) === false) {
        $score -= 20; // Meta açıklamada anahtar kelime yoksa puanı düşür
    }

    // Görsel ALT etiketleri kontrolü
    $images = get_attached_media('image', $post_id);
    foreach ($images as $image) {
        $alt_text = get_post_meta($image->ID, '_wp_attachment_image_alt', true);
        if (empty($alt_text)) {
            $score -= 10; // ALT etiketi olmayan her görsel için puanı düşür
        }
    }

    return $score;
}

// İç ve dış bağlantı analizini yapan fonksiyon
function mimoza_link_analysis($post_id) {
    $content = get_post_field('post_content', $post_id);
    
    // İç bağlantı (aynı siteye bağlantı)
    $site_url = get_site_url();
    $internal_links = substr_count($content, $site_url);
    
    // Dış bağlantı (başka sitelere bağlantı)
    preg_match_all('/<a\s+href="https?:\/\/(?!' . preg_quote($site_url, '/') . ')/i', $content, $external_links);
    $external_links_count = count($external_links[0]);

    $suggestion = "";

    // İç bağlantı önerisi
    if ($internal_links == 0) {
        $suggestion .= "Bu yazıda iç bağlantı bulunmuyor. İç bağlantı eklemeyi düşünebilirsiniz.<br>";
    }

    // Dış bağlantı önerisi
    if ($external_links_count == 0) {
        $suggestion .= "Bu yazıda dış bağlantı bulunmuyor. Dış bağlantı eklemeyi düşünebilirsiniz.<br>";
    } elseif ($external_links_count > 5) {
        $suggestion .= "Bu yazıda çok fazla dış bağlantı var. Dış bağlantıları azaltmayı düşünebilirsiniz.<br>";
    }

    return $suggestion;
}

// Başlık (H1-H6) ve anahtar kelime analizini yapan fonksiyon
function mimoza_seo_heading_keyword_analysis($post_id) {
    $content = get_post_field('post_content', $post_id);
    
    // H1 başlık kontrolü
    preg_match_all('/<h1.*?>.*?<\/h1>/', $content, $h1_matches);
    $h1_count = count($h1_matches[0]);

    // H2 başlık kontrolü
    preg_match_all('/<h2.*?>.*?<\/h2>/', $content, $h2_matches);
    $h2_count = count($h2_matches[0]);

    // Anahtar kelime kontrolü (İçerikteki anahtar kelime yoğunluğu)
    $focus_keyword = 'hedef anahtar kelime'; // Anahtar kelimeyi burada belirtebilirsin
    $keyword_count = substr_count(strtolower($content), strtolower($focus_keyword));

    $suggestions = "";

    // H1 başlık önerisi
    if ($h1_count == 0) {
        $suggestions .= "Bu yazıda H1 başlığı bulunmuyor. H1 başlığı eklemelisiniz.<br>";
    } elseif ($h1_count > 1) {
        $suggestions .= "Bu yazıda birden fazla H1 başlığı bulunuyor. Yalnızca bir H1 başlığı kullanmalısınız.<br>";
    }

    // H2 başlık önerisi
    if ($h2_count == 0) {
        $suggestions .= "Bu yazıda H2 başlığı bulunmuyor. H2 başlıkları ekleyerek içeriği daha iyi organize edebilirsiniz.<br>";
    }

    // Anahtar kelime yoğunluğu önerisi
    if ($keyword_count == 0) {
        $suggestions .= "Bu yazıda anahtar kelime hiç kullanılmamış. İçeriğe anahtar kelime eklemelisiniz.<br>";
    } elseif ($keyword_count > 10) {
        $suggestions .= "Anahtar kelime çok fazla kullanılmış. Anahtar kelime yoğunluğunu azaltmalısınız.<br>";
    }

    return $suggestions;
}

// Mevcut tabloya başlık, anahtar kelime ve link optimizasyon önerilerini ekleyelim
function mimoza_seo_link_list() {
    global $wpdb;

    // Tüm yazı ve sayfa linklerini sorgula
    $posts = $wpdb->get_results("
        SELECT ID, post_title, post_name, post_type 
        FROM $wpdb->posts 
        WHERE post_status = 'publish' 
        AND (post_type = 'post' OR post_type = 'page')
    ");

    if ($posts) {
        echo '<table class="widefat fixed">';
        echo '<thead><tr><th>ID</th><th>Başlık</th><th>URL</th><th>Meta Açıklaması</th><th>Durum</th><th>SEO Puanı</th><th>Başlık & Anahtar Kelime Önerileri</th><th>Link Önerileri</th><th>Güncelle</th></tr></thead>';
        echo '<tbody>';
        foreach ($posts as $post) {
            $post_url = get_permalink($post->ID);
            $meta_desc = get_post_meta($post->ID, '_yoast_wpseo_metadesc', true); // Meta açıklamayı çek
            $seo_score = mimoza_seo_score($post->ID); // SEO puanını hesapla
            $heading_keyword_suggestions = mimoza_seo_heading_keyword_analysis($post->ID); // Başlık ve anahtar kelime analizi yap
            $link_suggestions = mimoza_link_analysis($post->ID); // İç ve dış bağlantı analizi yap

            // Kırık link kontrolü
            $is_broken = mimoza_check_broken_link($post_url) ? 'Geçerli' : 'Kırık';

            // Meta açıklama formu ile birlikte tabloyu göster
            echo '<tr>';
            echo '<td>' . $post->ID . '</td>';
            echo '<td>' . $post->post_title . '</td>';
            echo '<td><a href="' . esc_url($post_url) . '" target="_blank">' . esc_html($post_url) . '</a></td>';
            echo '<td>';
            echo '<form method="POST" action="">';
            echo '<input type="hidden" name="post_id" value="' . $post->ID . '">';
            echo '<input type="text" name="meta_description" value="' . esc_attr($meta_desc) . '" style="width: 100%;">';
            echo '</td>';
            echo '<td>' . esc_html($is_broken) . '</td>'; // Kırık link durumu
            echo '<td>' . esc_html($seo_score) . '</td>'; // SEO Puanı
            echo '<td>' . $heading_keyword_suggestions . '</td>'; // Başlık ve anahtar kelime önerileri
            echo '<td>' . $link_suggestions . '</td>'; // Link önerileri
            echo '<td>';
            echo '<input type="submit" name="mimoza_update_meta" value="Güncelle" class="button-primary">';
            echo '</form>';
            echo '</td>';
            echo '</tr>';
        }
        echo '</tbody>';
        echo '</table>';
    } else {
        echo '<p>Hiçbir yazı veya sayfa bulunamadı.</p>';
    }
}
